@@ -23,20 +23,10 @@
  */
 package org.broadinstitute.dropseqrna.barnyard.digitalexpression.tools;
 
-import java.io.BufferedInputStream;
-import java.io.BufferedReader;
-import java.io.BufferedWriter;
-import java.io.File;
-import java.io.IOException;
-import java.io.OutputStreamWriter;
-import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.Collection;
-import java.util.Collections;
-import java.util.HashMap;
-import java.util.List;
-import java.util.Map;
-
+import htsjdk.samtools.util.CloserUtil;
+import htsjdk.samtools.util.IOUtil;
+import htsjdk.samtools.util.Log;
+import htsjdk.samtools.util.ProgressLogger;
 import org.apache.commons.collections4.CollectionUtils;
 import org.apache.commons.lang.ArrayUtils;
 import org.apache.commons.lang.StringUtils;
@@ -48,16 +38,17 @@ import org.broadinstitute.dropseqrna.matrixmarket.MatrixMarketWriter;
 import org.la4j.Matrix;
 import org.la4j.Vectors;
 import org.la4j.iterator.MatrixIterator;
+import org.la4j.iterator.VectorIterator;
 import org.la4j.matrix.SparseMatrix;
 import org.la4j.matrix.sparse.CRSMatrix;
+import org.la4j.vector.SparseVector;
 import org.la4j.vector.functor.VectorAccumulator;
-
-import htsjdk.samtools.util.CloserUtil;
-import htsjdk.samtools.util.IOUtil;
-import htsjdk.samtools.util.Log;
 import picard.util.BasicInputParser;
 import picard.util.TabbedInputParser;
 
+import java.io.*;
+import java.util.*;
+
 /**
  * This class is designed to hold a matrix of gene expression, addressable by gene name and cell barcode
  * This class supports reading and writing in the standard format for DGE data, merging of DGEMatrixes in place,
@@ -79,22 +70,29 @@ public class DGEMatrix {
 	// holds a map of the cell barcode to the position in the expression array
 	// this is consistent across all cells.
 	private final Map<String, Integer> cellBarcodeMap;
+	// Computed as necessary, and invalidate when appropriate
+	private List<String> cellBarcodeCache;
 
 	// the expression data in a matrix
 	private Matrix expressionMatrix;
 	private final double SPARSE_VALUE=0;
 
 	public DGEMatrix(final List<String> cellBarcodes, final List<String> geneNames, final double [] [] expressionMatrix) {
-		if (expressionMatrix.length!=geneNames.size())
-			throw new IllegalArgumentException("Rows of expression matrix [" + expressionMatrix.length +"] not equal to number of genes [" + geneNames.size()+ "]");
-		if (expressionMatrix[0].length!=cellBarcodes.size())
-			throw new IllegalArgumentException("Columns of expression matrix [" + expressionMatrix[0].length +"] not equal to number of cells [" + cellBarcodes.size()+ "]");
-
-		// add data
-		this.expressionMatrix = CRSMatrix.from2DArray(expressionMatrix);
-		this.geneMap = listToMap(geneNames);
-		this.cellBarcodeMap=listToMap(cellBarcodes);
+		this(cellBarcodes, geneNames, from2DCtorHelper(cellBarcodes, geneNames, expressionMatrix));
+	}
 
+	private static Matrix from2DCtorHelper(final List<String> cellBarcodes,
+										   final List<String> geneNames,
+										   final double [] [] expressionMatrix) {
+		if (expressionMatrix.length!=geneNames.size()) {
+			throw new IllegalArgumentException("Rows of expression matrix [" + expressionMatrix.length +
+					"] not equal to number of genes [" + geneNames.size() + "]");
+		}
+		if (expressionMatrix[0].length!=cellBarcodes.size()) {
+			throw new IllegalArgumentException("Columns of expression matrix [" + expressionMatrix[0].length +
+					"] not equal to number of cells [" + cellBarcodes.size() + "]");
+		}
+		return CRSMatrix.from2DArray(expressionMatrix);
 	}
 
 	/**
@@ -103,16 +101,14 @@ public class DGEMatrix {
 	 * @param cellBarcodes A list of cell barcodes that are columns of the matrix.
 	 */
 	public DGEMatrix(final List<String> cellBarcodes, final List<String> geneNames) {
-		// add data
-		this.expressionMatrix = CRSMatrix.zero(geneNames.size(), cellBarcodes.size());
-		this.geneMap = listToMap(geneNames);
-		this.cellBarcodeMap=listToMap(cellBarcodes);
+		this(cellBarcodes, geneNames, CRSMatrix.zero(geneNames.size(), cellBarcodes.size()));
 	}
 
 	private DGEMatrix(final List<String> cellBarcodes, final List<String> geneNames, final Matrix m) {
 		this.expressionMatrix = m;
 		this.geneMap = listToMap(geneNames);
 		this.cellBarcodeMap=listToMap(cellBarcodes);
+		this.cellBarcodeCache = new ArrayList<>(cellBarcodes);
 	}
 
 
@@ -173,9 +169,11 @@ public class DGEMatrix {
 	 *
 	 * @return a list of cell barcodes in this data set.
 	 */
-	// TODO: should these results be cached?
 	public List<String> getCellBarcodes () {
-		return (mapToList(this.cellBarcodeMap));
+		if (cellBarcodeCache == null) {
+			cellBarcodeCache = mapToList(this.cellBarcodeMap);
+		}
+		return Collections.unmodifiableList(cellBarcodeCache);
 	}
 
 	/**
@@ -189,11 +187,15 @@ public class DGEMatrix {
 		List<Integer> toRemove = new ArrayList<>();
 		for (String cell: cellBarcodes) {
 			Integer idx = this.cellBarcodeMap.get(cell);
-			if (idx!=null) {
-				toRemove.add(idx);
-				removeFromMap(cell, this.cellBarcodeMap);
-			}
+			if (idx!=null) 
+				toRemove.add(idx);							
 		}
+		// remove the indexes from the map
+		for (String cell: cellBarcodes) {
+			removeFromMap(cell, this.cellBarcodeMap);
+		}
+		cellBarcodeCache = null;
+		
 		Collections.sort(toRemove);
 		Collections.reverse(toRemove);
 		// if I remove elements from the end and work forward, I will not mess up if columns are filling in the gaps behind me.
@@ -313,6 +315,10 @@ public class DGEMatrix {
 			return (getExpressionDense(gene));
 	}
 
+	public double getExpression(final String gene, final String cellBarcode) {
+		return expressionMatrix.get(geneMap.get(gene), cellBarcodeMap.get(cellBarcode));
+	}
+
 	private double [] getExpressionSparse (final String gene) {
 		CRSMatrix thisM = (CRSMatrix) this.expressionMatrix;
 		Integer rowIdx = this.geneMap.get(gene);
@@ -354,6 +360,79 @@ public class DGEMatrix {
 		return this.expressionMatrix.toDenseMatrix().toArray();
 	}
 
+	public DGEMatrix merge (final DGEMatrix other) {
+		if (!CollectionUtils.intersection(this.getCellBarcodes(), other.getCellBarcodes()).isEmpty()) {
+			return mergeExpressionForCells(other);
+		} else {
+			return mergeDisjointCells(other);
+		}
+	}
+
+	/**
+	 * Merge DGEs that have different genes in each DGE.  List of cells may overlap, or may not.
+	 * @return a new DGEMatrix that is the merge of this and other.
+	 */
+	private DGEMatrix mergeExpressionForCells(DGEMatrix other) {
+		List<String> thisGenes = this.getGenes();
+		List<String> otherGenes = other.getGenes();
+		Collection<String> geneOverlap = CollectionUtils.intersection(thisGenes, otherGenes);
+		if (!geneOverlap.isEmpty()) {
+			throw new IllegalArgumentException("The two matrixes have overlapping genes, this should not happen:" +
+					geneOverlap.toString());
+		}
+		List<String> allGenes = new ArrayList<> (thisGenes);
+		CollectionUtils.addAll(allGenes, otherGenes);
+		Collections.sort(allGenes);
+		final ProgressLogger progressLogger = new ProgressLogger(log, 1000, "processed",
+				String.format("genes of %d", allGenes.size()));
+		// Determine which DGE has more cells.
+		final DGEMatrix bigger;
+		final DGEMatrix smaller;
+		if (this.getCellBarcodes().size() >= other.getCellBarcodes().size()) {
+			bigger = this;
+			smaller = other;
+		} else {
+			bigger = other;
+			smaller = this;
+		}
+		// Cells from bigger list go first, and then any that are in the smaller but not the bigger.
+		final List<String> allCellBarcodes = new ArrayList<>(bigger.getCellBarcodes());
+		// Create a map of the old to new index of columns in the DGE with fewer columns.
+		final Map<Integer, Integer> smallerColumnMap = new HashMap<>();
+		for (int column = 0; column < smaller.getCellBarcodes().size(); ++column) {
+			final String cell = smaller.getCellBarcodes().get(column);
+			int newColumn = allCellBarcodes.indexOf(cell);
+			if (newColumn == -1) {
+				newColumn = allCellBarcodes.size();
+				allCellBarcodes.add(cell);
+			}
+			smallerColumnMap.put(column, newColumn);
+		}
+		CRSMatrix m = new CRSMatrix(allGenes.size(), allCellBarcodes.size());
+		final Set<String> biggerGenesSet = new HashSet<>(bigger.getGenes());
+		for (int i = 0; i < allGenes.size(); ++i) {
+			final String gene = allGenes.get(i);
+			org.la4j.Vector outVec;
+			if (biggerGenesSet.contains(gene)) {
+				outVec = bigger.expressionMatrix.getRow(bigger.geneMap.get(gene));
+				// append zeroes as appropriate
+				outVec = outVec.copyOfLength(allCellBarcodes.size());
+			} else {
+				final double[] primVec = new double[allCellBarcodes.size()];
+				final VectorIterator it = smaller.expressionMatrix.getRow(smaller.geneMap.get(gene)).toSparseVector().nonZeroIterator();
+				while (it.hasNext()) {
+					final double val = it.next();
+					final int newIndex = smallerColumnMap.get(it.index());
+					primVec[newIndex] = val;
+				}
+				outVec = SparseVector.fromArray(primVec);
+			}
+			m.setRow(i, outVec);
+			progressLogger.record(gene, i);
+		}
+		return (new DGEMatrix(allCellBarcodes, allGenes, m));
+	}
+
 	/**
 	 * Merges two DGE matrixes together.  This retains all columns in both experiments, and adds rows that are missing in either data set.
 	 * The missing values are set to <missingValue>.
@@ -361,10 +440,11 @@ public class DGEMatrix {
 	 * @param other The other data set to merge with this one.
 	 * @return a new DGEMatrix containing the merge of the this with other.
 	 */
-	public DGEMatrix merge (final DGEMatrix other) {
+	public DGEMatrix mergeDisjointCells (final DGEMatrix other) {
 		Collection<String>overlap = CollectionUtils.intersection(this.getCellBarcodes(), other.getCellBarcodes());
-		if (overlap.size()>0)
+		if (!overlap.isEmpty()) {
 			throw new IllegalArgumentException("The two matrixes have overlapping cell barcodes, this should not happen:" + overlap.toString());
+		}
 
 		// merge the cell barcodes.
 		List <String> cellBarcodes = new ArrayList<>(this.getCellBarcodes());
