@@ -142,7 +142,7 @@ public class DetectBeadSubstitutionErrors extends CommandLineProgram{
 				GeneFunctionCommandLineBase.DEFAULT_GENE_NAME_TAG, GeneFunctionCommandLineBase.DEFAULT_GENE_STRAND_TAG,
                 GeneFunctionCommandLineBase.DEFAULT_GENE_FUNCTION_TAG, GeneFunctionCommandLineBase.DEFAULT_STRAND_STRATEGY,
                 GeneFunctionCommandLineBase.DEFAULT_LOCUS_FUNCTION_LIST, this.CELL_BARCODE_TAG, this.MOLECULAR_BARCODE_TAG,
-                this.READ_MQ, false, cellBarcodes, true);
+                this.READ_MQ, false, cellBarcodes, true, false);
 
         // get list of barcodes that have enough UMIs, and are not polyT biased.
         UMIsPerCellResult umiResult=getUMIsPerCell(umiIterator, this.MIN_UMIS_PER_CELL, this.UMI_BIAS_BASE, this.UMI_BIAS_THRESHOLD, null);
